
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"

#include <time.h>                           /* time() */
#include <fcntl.h>                          /* fcntl() */
#include <unistd.h>                         /* execv() pipe() usleep() pause() chown() */
#include <sys/wait.h>                       /* waitpid() sa */
#include <sys/ioctl.h>                      /* ioctl() */
#include <stdlib.h>                         /* free() exit() */
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <pwd.h>
#include <sys/stat.h>
#include <errno.h>


#include "../../src/initng_active_db.h"
#include "../../src/initng_global.h"
#include "../../src/initng_plugin_hook.h"
#include "../../src/initng_plugin_callers.h"
#include "../../src/initng_execute.h"
#include "../../src/initng_string_tools.h"
#include "../../src/initng_toolbox.h"
#include "../../src/initng_signal.h"
#include "../../src/initng_fork.h"

#include "initng_ash_launcher.h"

s_entry START_SCRIPT = { "start {", SCRIPT, 7 };
s_entry STOP_SCRIPT = { "stop {", SCRIPT, 6 };

static pid_t
script_initng_execute_fork_and_exec(process_h * process_to_exec, active_h * s,
                                    char *script, char *args);

static void ash_this(char *sh_code, active_h * s, char *args)
{
    D_("ash_this(%s);\n", s->name);
    if (args)
        W_("args %s ignored\n", args);
    W_("ash_this is ignoring arguments, and bash_this deals with them in the wrong way, fixitfixitfixitfixit!!\n");
    execute_ash(s->name, sh_code);

    F_("bash_this(): child died!\n EEEEEEEEEEEEEEEEEEEEEEERRRRRRRRRRRROOOOOOOOOORR!\n");
    /* exit 1, why???, to emit signal? */
    initng_free();
    _exit(1);
}                                           /* end fork_and_exec() */

int initng_ash_start(active_h * service)
{
    char *e = NULL;

    assert(service);

    /* WE ARE EXECUTING A START SCRIPT */
    if ((e = active_db_get_string(&START_SCRIPT, service)))
    {
        if (script_initng_execute_fork_and_exec
            (service->start_process, service, e,
             active_db_get_string(initng_sdt_find("start_args"), service)))
            return (TRUE);

        return (FAIL);
    }
    return (FALSE);
}

int initng_ash_stop(active_h * service)
{
    char *e = NULL;

    assert(service);

    /* WE ARE EXECUTING A START SCRIPT */
    if ((e = active_db_get_string(&STOP_SCRIPT, service)))
    {
        if (script_initng_execute_fork_and_exec
            (service->stop_process, service, e,
             active_db_get_string(initng_sdt_find("stop_args"), service)))
            return (TRUE);

        return (FAIL);
    }
    return (FALSE);

}


static pid_t
script_initng_execute_fork_and_exec(process_h * process_to_exec, active_h * s,
                                    char *script, char *args)
{

    /* called from inside the service directory, return the PID or 0 on error */

    /* This is the real service kicker */
    pid_t pid_fork;             /* pid got from fork() */

    assert(process_to_exec);
    assert(script);

    /* if this is child */
    if ((pid_fork = initng_fork(s, process_to_exec)) == 0)
    {
        /* execute code */
        ash_this(script, s, args);

        /* ash This should NOT return! */
        _exit(33);
    }

    /* close sides of the pipes we dont need */
    close(process_to_exec->stdout[1]);
    process_to_exec->stdout[1] = -1;

    /* save pid of fork */
    D_("FROM_FORK Forkstarted pid %i.\n", pid_fork);
    process_to_exec->pid = pid_fork;
    return pid_fork;

}

int module_init(const char *version)
{
    D_("initng_simple_plugin: module_init();\n");

    if (strcmp(version, INITNG_VERSION) != 0)
    {
        F_("This module, is compiled for \"%s\" version, and initng is compiled on \"%s\" version, wont load this module!\n", INITNG_VERSION, version);
        return (FALSE);
    }
    initng_sdt_add(&START_SCRIPT);
    initng_sdt_add(&STOP_SCRIPT);
    ash_initialize();
    W_("todo : Add after fork option to run a bash -x, add this to bash_launcher aswell in a service, flag debug_service?\n");
    /* To set -x in ash do : ash_setflag('x',TRUE); */

    initng_add_hook(STARTL, 50, &initng_ash_start);
    initng_add_hook(STOPL, 50, &initng_ash_stop);

    return (TRUE);
}

void module_unload(void)
{
    D_("initng_simple_plugin: module_unload();\n");
    initng_sdt_del(&START_SCRIPT);
    initng_sdt_del(&STOP_SCRIPT);

    initng_del_hook(STARTL, &initng_ash_start);
    initng_del_hook(STOPL, &initng_ash_stop);

}
