
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"

#include <time.h>                           /* time() */
#include <fcntl.h>                          /* fcntl() */
#include <unistd.h>                         /* execv() pipe() usleep() pause() chown() */
#include <sys/wait.h>                       /* waitpid() sa */
#include <sys/ioctl.h>                      /* ioctl() */
#include <stdlib.h>                         /* free() exit() */
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <pwd.h>
#include <sys/stat.h>
#include <errno.h>


#ifdef BUSYBOX
#include "../../busybox-1.00/include/busybox.h"
#include "../../busybox-1.00/include/libbb.h"
#endif

#include "../../src/initng_active_db.h"
#include "../../src/initng_global.h"
#include "../../src/initng_plugin_hook.h"
#include "../../src/initng_plugin_callers.h"
#include "../../src/initng_execute.h"
#include "../../src/initng_string_tools.h"
#include "../../src/initng_toolbox.h"
#include "../../src/initng_signal.h"
#include "../../src/initng_fork.h"

#include "initng_bash_launcher.h"

s_entry START_SCRIPT = { "start {", SCRIPT, 7 };
s_entry STOP_SCRIPT = { "stop {", SCRIPT, 6 };
s_entry DAEMON_SCRIPT = { "daemon {", SCRIPT, 8 };

static pid_t
script_initng_execute_fork_and_exec(process_h * process_to_exec, active_h * s,
                                    char *script, char *args);

static void bash_this(char *bash_code, active_h * s, char *args)
{
    D_("bash_this(%s);\n", s->name);
    /* Todo : move bash_this into initng_bash_launcher and parse args properly\n" */
    char **argtmp;
    argtmp = (char **) initng_calloc(8, sizeof(char *));
    argtmp[0] =
        (char *) initng_calloc(1, sizeof(char) * (15 + strlen(s->name)));
    strcpy(argtmp[0], "bash_helper[");
    strcat(argtmp[0], s->name);
    strcat(argtmp[0], "]");
    argtmp[1] = i_strdup("--noprofile");
    argtmp[2] = i_strdup("--norc");
    argtmp[3] = i_strdup("-c");
    argtmp[4] = bash_code;
    argtmp[5] = s->name;
    argtmp[6] = args;
    argtmp[7] = NULL;
    execve("/bin/bash", argtmp, new_environ(s));
    free(argtmp[0]);
    free(argtmp);
    F_("bash_this(): child died!\n EEEEEEEEEEEEEEEEEEEEEEERRRRRRRRRRRROOOOOOOOOORR!\n");
    /* exit 1, why???, to emit signal? */
    initng_free();
    _exit(1);
}                                           /* end fork_and_exec() */

int initng_bash_start(active_h * service)
{
    char *e = NULL;
    s_entry *START_ARGS = initng_sdt_find("start_args");

    assert(service);

    /* WE ARE EXECUTING A START SCRIPT */
    if ((e = active_db_get_string(&START_SCRIPT, service)))
    {
        char *args = NULL;

        if (START_ARGS)
            args = active_db_get_string(START_ARGS, service);

        if (script_initng_execute_fork_and_exec
            (service->start_process, service, e, args))
            return (TRUE);

        return (FAIL);
    }
    return (FALSE);
}

int initng_bash_daemon(active_h * service)
{
    char *e = NULL;
    s_entry *DAEMON_ARGS = initng_sdt_find("daemon_args");

    assert(service);

    /* WE ARE EXECUTING A START SCRIPT */
    if ((e = active_db_get_string(&DAEMON_SCRIPT, service)))
    {
        char *args = NULL;

        if (DAEMON_ARGS)
            args = active_db_get_string(DAEMON_ARGS, service);
        if (script_initng_execute_fork_and_exec
            (service->start_process, service, e, args))
            return (TRUE);

        return (FAIL);
    }
    return (FALSE);
}

int initng_bash_stop(active_h * service)
{
    char *e = NULL;
    s_entry *STOP_ARGS = initng_sdt_find("stop_args");

    assert(service);

    /* WE ARE EXECUTING A START SCRIPT */
    if ((e = active_db_get_string(&STOP_SCRIPT, service)))
    {
        char *args = NULL;

        if (STOP_ARGS)
            args = active_db_get_string(STOP_ARGS, service);
        if (script_initng_execute_fork_and_exec
            (service->stop_process, service, e, args))
            return (TRUE);

        return (FAIL);
    }
    return (FALSE);

}


static pid_t
script_initng_execute_fork_and_exec(process_h * process_to_exec, active_h * s,
                                    char *script, char *args)
{

    /* called from inside the service directory, return the PID or 0 on error */

    /* This is the real service kicker */
    pid_t pid_fork;             /* pid got from fork() */

    assert(process_to_exec);
    assert(script);

    if ((pid_fork = initng_fork(s, process_to_exec)) == 0)
    {
        /* execute code */
        bash_this(script, s, args);

        /* Bash This should NOT return! */
        _exit(33);
    }

    /* close sides of the pipes we dont need */
    close(process_to_exec->stdout[1]);
    process_to_exec->stdout[1] = -1;

    /* save pid of fork */
    D_("FROM_FORK Forkstarted pid %i.\n", pid_fork);
    process_to_exec->pid = pid_fork;
    return pid_fork;

}

int module_init(const char *version)
{
    D_("initng_simple_plugin: module_init();\n");
    if (strcmp(version, INITNG_VERSION) != 0)
    {
        F_("This module, is compiled for \"%s\" version, and initng is compiled on \"%s\" version, wont load this module!\n", INITNG_VERSION, version);
        return (FALSE);
    }

    initng_sdt_add(&START_SCRIPT);
    initng_sdt_add(&STOP_SCRIPT);
    initng_sdt_add(&DAEMON_SCRIPT);

    initng_add_hook(STARTL, 50, &initng_bash_start);
    initng_add_hook(STOPL, 50, &initng_bash_stop);
    initng_add_hook(STARTL, 50, &initng_bash_daemon);
    return (TRUE);
}

void module_unload(void)
{
    D_("initng_simple_plugin: module_unload();\n");
    initng_sdt_del(&START_SCRIPT);
    initng_sdt_del(&STOP_SCRIPT);
    initng_sdt_del(&DAEMON_SCRIPT);

    initng_del_hook(STARTL, &initng_bash_start);
    initng_del_hook(STOPL, &initng_bash_stop);
    initng_del_hook(STARTL, &initng_bash_daemon);

}
