
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"

#include <time.h>                           /* time() */
#include <fcntl.h>                          /* fcntl() */
#include <unistd.h>                         /* execv() pipe() usleep() pause() chown() */
#include <sys/wait.h>                       /* waitpid() sa */
#include <sys/ioctl.h>                      /* ioctl() */
#include <stdlib.h>                         /* free() exit() */
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <pwd.h>
#include <sys/stat.h>
#include <errno.h>


#include "../../src/initng_active_db.h"
#include "../../src/initng_global.h"
#include "../../src/initng_plugin_hook.h"
#include "../../src/initng_plugin_callers.h"
#include "../../src/initng_execute.h"
#include "../../src/initng_string_tools.h"
#include "../../src/initng_toolbox.h"
#include "../../src/initng_common.h"
#include "../../src/initng_signal.h"
#include "../../src/initng_fork.h"

#include "initng_execve_launch.h"

s_entry EXECVE = { "execve", STRING, 6 };
s_entry EXECVE_ARGS = { "execve_args", STRING, 11 };



int initng_execve(active_h * service)
{
    char *e = NULL;
    char *a = NULL;
    char **args = NULL;

    assert(service);
    /* WE ARE EXECUTING A START FILE */
    D_("Checking for execve flag...\n");
    if ((e = active_db_get_string(&EXECVE, service)))
    {

        struct stat test;

        D_("execve flag is: \"%s\"..\n");
        if (stat(e, &test) == -1)
        {
            F_("Stat failed on %s\n", e);
            return (FAIL);
        }

        if ((a = active_db_get_string(&EXECVE_ARGS, service)))
        {
            int argc = 0;

            D_("execve_args: %s\n", a);
            args = split_n(a, &argc, 2, 1);


        }
        else
        {
            D_("execve_args: empty\n");
            args = initng_calloc(2, sizeof(char *));    /* alloc a null */
            args[1] = '\0';                 /* set to null */
        }

        args[0] = e;

        D_("Doing an execve to, %s %s !\n", e, a ? a : "none");
        /* This will replace initng with this program, as pid no 1 */
        execve(e, args, environ);

        F_("execve failed, errno: %i ..\n", errno);
        return (FAIL);
    }

    return (FALSE);
}

int module_init(const char *version)
{
    D_("module_init();\n");
    if (strcmp(version, INITNG_VERSION) != 0)
    {
        F_("This module, is compiled for \"%s\" version, and initng is compiled on \"%s\" version, wont load this module!\n", INITNG_VERSION, version);
        return (FALSE);
    }

    initng_add_hook(STARTL, 90, &initng_execve);

    initng_sdt_add(&EXECVE);
    initng_sdt_add(&EXECVE_ARGS);

    return (TRUE);
}

void module_unload(void)
{
    D_("module_unload();\n");

    initng_sdt_del(&EXECVE);
    initng_sdt_del(&EXECVE_ARGS);

    initng_del_hook(STARTL, &initng_execve);

}
