
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"
#define _GNU_SOURCE
#include <stdio.h>
#include <alloca.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <assert.h>
#include <sys/types.h>
#include <dirent.h>                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                /*#include <dirent.h> */     /* opendir() closedir() */

#include "../../src/initng_string_tools.h"
#include "../../src/initng_service_cache.h"
#include "../../src/initng_global.h"
#include "../../src/initng_open_read_close.h"
#include "../../src/initng_toolbox.h"
#include "../../src/initng_plugin_hook.h"

#include "initng_i_parser.h"

static service_h *parse_file(char *filename, char *watch_for);
static int set_parser(s_entry * type, char **value, service_h * from_service);
static int unset_parser(s_entry * type, char **value,
                        service_h * from_service);
static int string_parser(s_entry * type, char **value,
                         service_h * from_service);
static int strings_parser(s_entry * type, char **value,
                          service_h * from_service);
static int script_parser(s_entry * type, char **value,
                         service_h * from_service);
static int int_parser(s_entry * type, char **value, service_h * from_service);




static int set_parser(s_entry * type, char **value
                      __attribute__ ((unused)), service_h * from_service)
{
    assert(from_service);
    assert(*value);
    assert(value);

    D_("set_parser(%s,s,%s);\n", type->opt_name, from_service->name);
    service_db_set(type, from_service);
    return (TRUE);
}

static int unset_parser(s_entry * type, char **value __attribute__ ((unused)),
                        service_h * from_service)
{
    assert(from_service);
    assert(*value);
    assert(value);

    D_("unset_parser(%s,s,%s);\n", type->opt_name, from_service->name);
    service_db_remove(type, from_service);
    return (TRUE);
}

static int string_parser(s_entry * type, char **value,
                         service_h * from_service)
{
    char *to;

    assert(from_service);
    if (!(*value))
        return (FALSE);
    D_("string_parser(%s,s,%s);\n", type->opt_name, from_service->name);
    if ((to = st_dup_line(value)))
    {
        service_db_set_string(type, from_service, to);
        return (TRUE);
    }
    return (FALSE);
}

static int int_parser(s_entry * type, char **value, service_h * from_service)
{
    char *to;

    assert(from_service);
    if (!(*value))
        return (FALSE);
    D_("string_parser(%s,s,%s);\n", type->opt_name, from_service->name);
    if ((to = st_dup_line(value)))
    {
        service_db_set_int(type, from_service, atoi(to));
        if (to)
            free(to);
        return (TRUE);
    }
    return (FALSE);
}

static int strings_parser(s_entry * type, char **value,
                          service_h * from_service)
{
    char *to = NULL;

    assert(from_service);
    if (!(*value))
        return (FALSE);
    D_("string_parser(%s,s,%s);\n", type->opt_name, from_service->name);

    while ((to = st_dup_next_word(value)))
    {
        service_db_set_another_string(type, from_service, to);
        st_jump_spaces(value);
    }
    return (TRUE);
    /*return(FALSE); */
}

static int script_parser(s_entry * type, char **value,
                         service_h * from_service)
{
    int stack_length = 0;

    assert(from_service);
    D_("script_parser(%s,s,%s);\n", type->opt_name, from_service->name);
    stack_length = chars_to_end_of_this_stack(*value);

    service_db_set_string(type, from_service,
                          i_strndup(*value, stack_length));
    (*value) = (*value) + stack_length;
    return (TRUE);
}

static int parse_opt(char **where, service_h * srv)
{
    int i;
    int w_l = 0;

    /* jump forward all spaces */
    st_jump_spaces(where);

    /* count number of chars this option name have */
    while ((*where)[w_l] && (*where)[w_l] != '\t' &&
           (*where)[w_l] != '\n' && (*where)[w_l] != ';'
           && (*where)[w_l] != '=')
        w_l++;

    /* strip ' ' from the end */
    while ((*where)[w_l - 1] == ' ')
        w_l--;

    if (!w_l)
    {
        F_("option name to short!\n");
        return (FALSE);
    }

    for (i = 0; g.option_table[i]; i++)
    {
        D_("this option_table[%i] has %i str_len.\n", i,
           g.option_table[i]->str_len);
        if (g.option_table[i]->opt_name &&
            w_l == g.option_table[i]->str_len &&
            strncasecmp(g.option_table[i]->opt_name, *where, w_l) == 0)
        {
            D_("parse_opt(%s): option \"%s\" value at option no %i.\n",
               srv->name, g.option_table[i]->opt_name, i);
            *where = *where + w_l;
            /* jump forward all spaces */
            st_jump_spaces(where);
            if ((*where)[0])
            {
                switch (g.option_table[i]->opt_type)
                {
                    case STRING:
                        return (string_parser(g.option_table[i], where, srv));
                    case STRINGS:
                        return (strings_parser
                                (g.option_table[i], where, srv));
                    case SCRIPT:
                        return (script_parser(g.option_table[i], where, srv));
                    case SET:
                        return (set_parser(g.option_table[i], where, srv));
                    case UNSET:
                        return (unset_parser(g.option_table[i], where, srv));
                    case INT:
                        return (int_parser(g.option_table[i], where, srv));
                    case U_D_T:
                        break;
                }
            }
        }

    }

    /* Go here if there are errors! */
    {
        char *line = i_strndup(*where, strcspn(*where, "\n\0"));

        F_("parse_opt(%s): line: \"%s\"!\n", srv->name, line);
        free(line);
    }
    F_("sleeping 2 seconds......\n");
    sleep(2);
    return (FALSE);
}

static service_h *parse_service_line(char **to_parse, char *watch_for,
                                     service_h * father, e_type type,
                                     char *filename)
{
    char *name = NULL;
    char *father_name = NULL;
    service_h *new_service = NULL;
    service_h *service_to_return = NULL;
    service_h *duplicate = NULL;

    assert(to_parse);
    assert(*to_parse);
    assert(watch_for);

    D_("parse_service_line(*s*,%s);\n", watch_for);

    /* service test : class { */
    /*        |               */
    /* jump forward on spaces */
    st_jump_spaces(to_parse);

    /* service test : class { */
    /*         |              */

    /* fetch name */
    if (!(name = st_dup_next_word(to_parse)))
    {
        F_("Did not got name!\n");
        return (NULL);
    }
    /* service test : class { */
    /*             |          */

    D_("parse_service_line(s,%s): service name: \"%s\"\n", watch_for, name);

    /* jump forward on spaces */
    st_jump_spaces(to_parse);

    /* service test : class { */
    /*              |         */
    if ((*to_parse)[0] == ':')
    {
        (*to_parse)++;
        /* service test : class { */
        /*               |        */

        /* jump forward on spaces */
        st_jump_spaces(to_parse);
        /* service test : class { */
        /*                |       */

        father_name = st_dup_next_word(to_parse);
        if (father_name)
        {
            D_("Father is: %s, set from \"server : father\"\n", father_name);
        }

        /* service test : class { */
        /*                     |  */
        /* jump forward on spaces */
        st_jump_spaces(to_parse);

    }

    /* service test : class { */
    /*                      | */
    if ((*to_parse)[0] != '{')
    {
        /* something is wrong */
        F_("error parsing, expected something else.\n");
        return (NULL);
    }

    /* jump to first char after start tag and begin from there. */
    (*to_parse)++;
    /* service test : class { */
    /*                       | */

    /* create a service entry - if there exits an duplicate, merge with this one */
    if ((duplicate = service_db_find_by_name(name)))
        service_db_copy(&new_service, name, duplicate);
    else
        service_db_new(&new_service, name, type);

    /* set the father to the service */
    if (father_name)
    {
        new_service->father_name = father_name;
        /* father will be service_db_find_by_name(father_name) when needed */
    }
    else if (father && father->name)
    {
        new_service->father_name = father->name;
        new_service->father = father;
    }

    if (!new_service)
    {
        F_("parse_service_line(s,s): unable to allocate new service!\n");
        free(name);
        return (NULL);
    }

    if (duplicate)
    {
        /* Service has same name as old service, remove the old one */
        D_("Removing old service for %s\n", new_service->name);
        service_db_free(duplicate);
    }

    /* set type if not set. */
    if (father)
        new_service->type = type;

    /* skip lines with '#' */
    st_jump_to_valid_line(to_parse);

    /* carry on until segment stop or eof */
    while (*to_parse[0] != '\0' && *to_parse[0] != '}')
    {
        service_h *tmp = NULL;

        /* if this row has keyword service */
        if (st_cmp(to_parse, "service"))
            if ((tmp =
                 parse_service_line(to_parse, watch_for, NULL, TYPE_SERVICE,
                                    filename)))
                service_to_return = tmp;

        /* if this row has keyword class */
        if (st_cmp(to_parse, "class"))
            if ((tmp =
                 parse_service_line(to_parse, watch_for, NULL, TYPE_CLASS,
                                    filename)))
                service_to_return = tmp;

        /* parse line - search for keywords in g.option_table */
        if (!parse_opt(to_parse, new_service))
        {
            /* jump to end of stack */
            *to_parse = *to_parse + chars_to_end_of_this_stack(*to_parse);
            st_jump_to_next_row(to_parse);
            return (NULL);
        }

        /* jump to next row */
        st_jump_to_next_row(to_parse);
    }

    service_db_set_string(&FROM_FILE, new_service, i_strdup(filename));
    /* add to service db */
    if (!service_db_add(new_service))
    {
        F_("can't add service to service db!\n");
        if (new_service->name)
            free(new_service->name);
        if (new_service->data)
            free(new_service->data);
        if (new_service)
            free(new_service);
        return (NULL);
    }

    /* check if this is the service we are looking for. */
    if (st_service_cmp(watch_for, new_service->name))
        return (new_service);

    /* if we got a service from a under service */
    if (service_to_return)
        return (service_to_return);
    /* return null if not */
    return (NULL);
}

/* parse a file for services */
static service_h *parse_file(char *filename, char *watch_for)
{
    char *file, *orig;
    service_h *service_to_return = NULL;
    service_h *tmp = NULL;

    assert(filename);
    assert(watch_for);

    D_("parse_file(%s);\n", filename);

    /* read config file */
    if (!open_read_close(filename, &orig))
    {
        D_("Can't open config file %s.\n", filename);
        return (NULL);
    }
    file = orig;

    /* skip lines starting with '#' */
    st_jump_to_valid_line(&file);

    /* main parse line by line loop */
    while (file[0] != '\0')
    {

        /* if this row has keyword service */
        if (st_cmp(&file, "service"))
            if ((tmp =
                 parse_service_line(&file, watch_for, NULL, TYPE_SERVICE,
                                    filename)))
                service_to_return = tmp;

        /* if this row has keyword class */
        if (st_cmp(&file, "class"))
            if ((tmp =
                 parse_service_line(&file, watch_for, NULL, TYPE_CLASS,
                                    filename)))
                service_to_return = tmp;

        /* jump file pointer to first char next row */
        st_jump_to_next_row(&file);
    }                                       /* end while */

    free(orig);
#ifdef DEBUG_PRINT
    print_all();
#endif
    /*exit_initng(1); */

    tmp = service_db_find_by_name(watch_for);
    if (tmp)
        return (tmp);

    if (service_to_return)
        return (service_to_return);
    return (NULL);
}

/* Load a service from a process_name or process_path */
service_h *initng_i_parser(char *process_name)
{
    char filetoparse[200];
    char *tmp = NULL;
    service_h *got_serv = NULL;

    assert(process_name);
    D_("initng_i_parser(%s);\n", process_name);
    /* example  INITNG_ROOT=/etc/initng process_name="deamon/sshd INITNG_EXT=".i" */
    /* TRY NO 1, try load   "/etc/initng/daemon/sshd.i" */
    /* TRY NO 2, try load   "/etc/initng/daemon/sshd/sshd.i" */
    /* TRY NO 3, try load   "/etc/initng/daemon/sshd/default.i" */
    /* TRY NO 4, try load   "/etc/initng/daemon/default.i" */
    /* TRY NO 5, try load   "/etc/initng/daemon/daemon.i */
    /* TRY NO 6, try load   "/etc/initng/sshd.i" */
    /* TRY NO 7, try load   "/etc/initng/daemon.i */

    /* TRY NO 1, try load   "/etc/initng/daemon/sshd/sshd.i" */
    strcpy(filetoparse, INITNG_ROOT "/");
    strcat(filetoparse, process_name);
    strcat(filetoparse, "/");
    strcat(filetoparse, st_strip_path(process_name));
    strcat(filetoparse, INITNG_EXT);
    if ((got_serv = parse_file(filetoparse, process_name)))
        return (got_serv);

    /* TRY NO 2, try load   "/etc/initng/daemon/sshd.i" */
    strcpy(filetoparse, INITNG_ROOT "/");
    strcat(filetoparse, process_name);
    strcat(filetoparse, INITNG_EXT);
    if ((got_serv = parse_file(filetoparse, process_name)))
        return (got_serv);

    /* TRY NO 3, try load   "/etc/initng/daemon/sshd/default.i" */
    strcpy(filetoparse, INITNG_ROOT "/");
    strcat(filetoparse, process_name);
    strcat(filetoparse, "/default" INITNG_EXT);
    if ((got_serv = parse_file(filetoparse, process_name)))
        return (got_serv);

    /* TRY NO 4, try "/etc/initng/daemon/default.i" */
    tmp = st_get_path(process_name);
    strcpy(filetoparse, INITNG_ROOT "/");
    strcat(filetoparse, tmp);
    strcat(filetoparse, "/default" INITNG_EXT);
    free(tmp);
    if ((got_serv = parse_file(filetoparse, process_name)))
        return (got_serv);

    /* TRY NO 5, try "/etc/initng/daemon/daemon.i" */
    tmp = st_get_path(process_name);
    strcpy(filetoparse, INITNG_ROOT "/");
    strcat(filetoparse, tmp);
    strcat(filetoparse, "/");
    strcat(filetoparse, tmp);
    strcat(filetoparse, INITNG_EXT);
    free(tmp);
    if ((got_serv = parse_file(filetoparse, process_name)))
        return (got_serv);

    /* TRY NO 6, try "/etc/initng/sshd.i" */
    strcpy(filetoparse, INITNG_ROOT "/");
    strcat(filetoparse, st_strip_path(process_name));
    strcat(filetoparse, INITNG_EXT);
    if ((got_serv = parse_file(filetoparse, process_name)))
        return (got_serv);

    /* TRY NO 7, try "/etc/initng/daemon.i" */
    tmp = st_get_path(process_name);
    strcpy(filetoparse, INITNG_ROOT "/");
    strcat(filetoparse, tmp);
    strcat(filetoparse, INITNG_EXT);
    free(tmp);
    if ((got_serv = parse_file(filetoparse, process_name)))
        return (got_serv);



    D_("initng_load_process(%s): got \"NULL\"\n", process_name);
    return (NULL);
}


int module_init(const char *version)
{

    D_("i_parser: module_init();\n");
    if (strcmp(version, INITNG_VERSION) != 0)
    {
        F_("This module, is compiled for \"%s\" version, and initng is compiled on \"%s\" version, wont load this module!\n", INITNG_VERSION, version);
        return (FALSE);
    }

    return (initng_add_hook(PARSERS, 50, &initng_i_parser));

}

void module_unload(void)
{
    D_("i_parser: module_unload();\n");
    initng_del_hook(PARSERS, &initng_i_parser);
}
