
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 neuron <neuron@hollowtube.mine.nu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "../../src/initng.h"

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <signal.h>
#include <errno.h>
#include <assert.h>
#include <string.h>
#include "../../src/initng_handler.h"
#include "../../src/initng_global.h"
#include "../../src/initng_common.h"
#include "../../src/initng_plugin_hook.h"

#include "initng_pidfile.h"

int its_ok = 0;
int check_if_done(active_h * s)
{
    assert(s);
    assert(s->name);

    if (its_ok)
        return (TRUE);
    /* if there is a service, that claims that it is running */
    if (s->a_status == RUNNING && active_db_is(&PIDFILE, s))
    {
        /* its not up yet */
        D_(" This service is not up, it have to wait for pidfile first!\n");
        mark_service(s, WAITING_FOR_PID_FILE);
        return (TRUE);
    }
    return (TRUE);
}

s_entry PIDFILE = { "pid_file", STRING, 8 };
int handle_killed(active_h * s, process_h * p)
{
    int fd = 0;
    int len = 0;
    int newpid = 0;
    char buf[21];
    char *pidfile;

    /* check if service is waiting for a pidfile */
    if (s->a_status != WAITING_FOR_PID_FILE)
    {
        D_("This service is not waiting for a pidfile, skipping.\n");
        return (FALSE);
    }

    /* check if the start_process exists */
    if (!s->start_process)
    {
        F_("Cant find start_process, FATAL!\n");
        return (FALSE);
    }


    /* We only override start behavior, and only if pidfile is set */
    if (!(pidfile = active_db_get_string(&PIDFILE, s)))
    {
        D_("Service dont use pidfile!\n");
        return (FALSE);
    }


    D_("service %s uses pidfile \"%s\"\n", s->name, pidfile);
    pidfile = fix_variables(pidfile, s);
    D_("Also known as \"%s\"\n", pidfile);


    /* open pid file */
    fd = open(pidfile, O_RDONLY);

    /* If we cant open pidfile, this is bad */
    if (fd == -1)
    {
        F_("Fatal, unable to open pidfile: %s, %s\n", pidfile, strerror(errno));
        return (FALSE);
    }

    /* Read data from buffer */
    len = read(fd, buf, 20);
    close(fd);
    if (len < 1)
    {
        F_("Read 0 from %s\n", pidfile);
        free(pidfile);
        return (FALSE);
    }

    /* remove last newline */
    if (buf[len - 1] == '\n')
        buf[len - 1] = 0;
    else
        buf[len] = 0;

    /* Try to convert pid to int */
    newpid = atoi(buf);
    if (newpid < 2)
    {
        F_("Failed to convert pid %s (%s) to int\n", buf, pidfile);
        free(pidfile);
        return (FALSE);
    }

    /* Check if the old pid == newpid) */
    if (newpid == p->pid)
    {
        D_("newpid == p->pid");
        free(pidfile);
        return (FALSE);
    }


    /* See if it's a valid pid */
    if (kill(newpid, 0) < 0 && (errno == ESRCH))
    {
        F_("Pid %d found (%s), but is dead\n", newpid, pidfile);
        free(pidfile);
        return (FALSE);
    }

    /* Finally set it!\n */
    D_("Pid updated for %s, from %d to %d.\n", s->name, p->pid, newpid);
    p->pid = newpid;

    its_ok = 1;
    mark_service(s, RUNNING);
    its_ok = 0;

    free(pidfile);
    return (TRUE);
}

int module_init(const char *version)
{
    D_("module_init();\n");
    if (strcmp(version, INITNG_VERSION) != 0)
    {
        F_("This module, is compiled for \"%s\" version, and initng is compiled on \"%s\" version, wont load this module!\n", INITNG_VERSION, version);
        return (FALSE);
    }

    its_ok = 0;
    initng_sdt_add(&PIDFILE);

    initng_add_hook(HANDLE_KILLED, 10, &handle_killed);
    initng_add_hook(WATCHERS, 10, &check_if_done);

    return (TRUE);
}

void module_unload(void)
{
    D_("module_unload();\n");
    initng_sdt_del(&PIDFILE);
    initng_del_hook(WATCHERS, &check_if_done);
    initng_del_hook(HANDLE_KILLED, &handle_killed);
}
