
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"

#include <time.h>                           /* time() */
#include <fcntl.h>                          /* fcntl() */
#include <unistd.h>                         /* execv() pipe() usleep() pause() chown() */
#include <sys/wait.h>                       /* waitpid() sa */
#include <sys/ioctl.h>                      /* ioctl() */
#include <stdlib.h>                         /* free() exit() */
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <pwd.h>
#include <sys/stat.h>

#include "../../src/initng_active_db.h"
#include "../../src/initng_global.h"
#include "../../src/initng_plugin_hook.h"
#include "../../src/initng_plugin_callers.h"
#include "../../src/initng_execute.h"
#include "../../src/initng_string_tools.h"
#include "../../src/initng_toolbox.h"
#include "../../src/initng_common.h"
#include "../../src/initng_signal.h"
#include "../../src/initng_fork.h"
/*
   #include "initng_global.h"
   #include "initng_process_db.h"
   #include "main.h"
   #include "initng_string_tools.h"
   #include "initng_execute.h"
   #include "bash_executor.h"
   #include "initng_active_db.h"
   #include "initng_load_module.h"
   #include "initng_toolbox.h" */

#include "initng_simple_launcher.h"

s_entry DAEMON = { "daemon", STRINGS, 6 };
s_entry DAEMON_ARGS = { "daemon_args", STRING, 11 };
s_entry START = { "start", STRINGS, 5 };
s_entry START_ARGS = { "start_args", STRING, 10 };
s_entry STOP = { "stop", STRINGS, 4 };
s_entry STOP_ARGS = { "stop_args", STRING, 9 };


static pid_t
initng_execute_fork_and_exec(process_h * process_to_exec, active_h * s,
                             char *e, char *ea);

int initng_s_start(active_h * service)
{
    char *e = NULL;
    char *t = NULL;

    assert(service);
    /* WE ARE EXECUTING A START FILE */
    if (active_db_is(&START, service))
    {
        while ((e = active_db_get_next_string(&START, service, e)))
        {

            struct stat test;

            t = fix_variables(e, service);
            if (stat(t, &test) == -1)
            {
                F_("Stat failed on %s\n", t);
                free(t);
                continue;

            }


            if (initng_execute_fork_and_exec
                (service->start_process, service, t,
                 active_db_get_string(&START_ARGS, service)))
            {
                free(t);
                return (TRUE);
            }
            D_("%s did not work\n", t);
            free(t);
        }
        return (FAIL);
    }

    if (active_db_is(&DAEMON, service))
    {
        while ((e = active_db_get_next_string(&DAEMON, service, e)))
        {

            struct stat test;

            t = fix_variables(e, service);

            if (stat(t, &test) == -1)
            {
                D_("Stat failed on start %s, file dont exits, trying next if any.\n", t);
                free(t);
                continue;
            }

            if (initng_execute_fork_and_exec
                (service->start_process, service, t,
                 active_db_get_string(&DAEMON_ARGS, service)))
            {
                free(t);
                return (TRUE);
            }

            D_("start %s did not work, trying next if any.\n", t);
            free(t);
        }
        return (FAIL);

    }
    return (FALSE);
}

int initng_s_stop(active_h * service)
{
    char *e = NULL;
    char *t = NULL;

    assert(service);


    /* WE ARE EXECUTING A STOP SCRIPT */
    if (active_db_is(&STOP, service))
    {
        while ((e = active_db_get_next_string(&STOP, service, e)))
        {

            struct stat test;

            t = fix_variables(e, service);

            if (stat(t, &test) == -1)
            {
                D_("Stat failed on stop %s\n", t);
                free(t);
                continue;
            }


            if (initng_execute_fork_and_exec
                (service->stop_process, service, t,
                 active_db_get_string(&STOP_ARGS, service)))
            {
                free(t);
                return (TRUE);
            }
            D_("stop %s did not work\n", t);
            free(t);
        }
        return (FAIL);

    }
    return (FALSE);
}


static pid_t
initng_execute_fork_and_exec(process_h * process_to_exec, active_h * s,
                             char *e, char *ea)
{

    /* called from inside the service directory, return the PID or 0 on error */

    /* This is the real service kicker */
    pid_t pid_fork;             /* pid got from fork() */
    char **argv = 0;            /* Arguments */
    int argc;                   /* number of arguments */

    assert(process_to_exec);
    assert(e);

    if ((pid_fork = initng_fork(s, process_to_exec)) == 0)
    {
        char *exec = NULL;
        char *exec_args = NULL;

        ea = fix_variables(ea, s);

        exec = e;
        exec_args = ea;

        D_("FROM_FORK initng_execute_fork_and_exec(%s,%s);\n", e, ea);

        /* get args from service database */
        if (exec_args)
        {
            D_("FROM_FORK initng_execute_fork_and_exec(%s): start args: %s\n",
               exec, exec_args);

            /* if we only vfork, we need to copy this string because split destroys it */
            /*temp=i_strdup(process_to_exec->exec_args); */
            /* split args */
            argv = split_n(exec_args, &argc, 2, 1);

            /* i dont know about this */
            if (!argv[argc - 1])
            {
                argv[argc - 1] = '\0';      /* the last */
            }
            else
            {
                argv[argc] = '\0';          /* must be null */
            }
        }
        else
        {
            argv = initng_calloc(2, sizeof(char *));    /* alloc a null */
            argv[1] = '\0';                 /* set to null */
        }

        /* set argv[0] */
        argv[0] = exec;


        {
            int i;

            for (i = 0; argv[i]; i++)
                D_("i.e.f.a.e.(): argv[%i]=\"%s\"\n", i, argv[i]);
        }


        /* FINALLY EXECUTE *//* execve replaces the running process */
        execve(argv[0], argv, new_environ(s));

        /* Will never get here if execve succeeded */
        F_("EEEEEEEEEEEEEEEEEEERRRRRRRRRRRRRRRRRRRRRRRRRRROOOOOOOOOOOOOOOOOOOOOOOOOORRRRRRRRRRRRRRR!!!!!!!!!\n");
        F_("Cant execute source %s, \"%s\" !\n", e, ea);

        /* free variable, good for making valgrind quit */
        free(argv[0]);
        free(argv);
        free(exec_args);

        /* clean up */
        initng_free();

        /* exit 1, why???, to emit signal? */
        _exit(1);
    }

    /* save pid of fork */
    D_("FROM_FORK Forkstarted pid %i.\n", pid_fork);
    process_to_exec->pid = pid_fork;
    return pid_fork;

    /* if to test want to lock this up util fork is done ...   waitpid(pid_fork,0,0); */
}                                           /* end fork_and_exec() */


int module_init(const char *version)
{
    D_("initng_simple_plugin: module_init();\n");
    if (strcmp(version, INITNG_VERSION) != 0)
    {
        F_("This module, is compiled for \"%s\" version, and initng is compiled on \"%s\" version, wont load this module!\n", INITNG_VERSION, version);
        return (FALSE);
    }

    initng_add_hook(STARTL, 40, &initng_s_start);
    initng_add_hook(STOPL, 40, &initng_s_stop);

    initng_sdt_add(&DAEMON);
    initng_sdt_add(&DAEMON_ARGS);
    initng_sdt_add(&START);
    initng_sdt_add(&START_ARGS);
    initng_sdt_add(&STOP);
    initng_sdt_add(&STOP_ARGS);

    return (TRUE);
}

void module_unload(void)
{
    D_("initng_simple_plugin: module_unload();\n");

    initng_sdt_del(&DAEMON);
    initng_sdt_del(&DAEMON_ARGS);
    initng_sdt_del(&START);
    initng_sdt_del(&START_ARGS);
    initng_sdt_del(&STOP);
    initng_sdt_del(&STOP_ARGS);

    initng_del_hook(STARTL, &initng_s_start);
    initng_del_hook(STOPL, &initng_s_stop);

}
