
/* Initng, a 6next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"

#define _GNU_SOURCE
#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "../../src/initng_toolbox.h"
#include "../../src/initng_global.h"
#include "../../src/initng_service_cache.h"
#include "../../src/initng_active_db.h"


#include "print_service.h"

static void service_db_print_u(service_h * s, FILE * fd, int intro);
static void active_db_print_u(active_h * s, FILE * fd, int intro);

/* P R I N T   S E R V I C E */

/* some stdout */
void service_db_print(service_h * s, FILE * fd)
{
    service_db_print_u(s, fd, TRUE);
    return;
}

void active_db_print(active_h * s, FILE * fd)
{
    active_db_print_u(s, fd, TRUE);
    return;
}

static void service_db_print_u(service_h * s, FILE * fd, int intro)
{
    /*data path */
    s_data *tmp = s->data;

    assert(s);
    assert(s->name);

    /* try calculate out how mutch memory we wanna allocate
       that we will send */

    if (intro)
        switch (s->type)
        {
            case TYPE_SERVICE:
                fprintf(fd, "\n#  service  \"%s", s->name);
                if (s->father_name)
                    fprintf(fd, " : %s", s->father_name);
                fprintf(fd, "\"\n");
                break;
            case TYPE_CLASS:
                fprintf(fd, "\n#  class    \"%s\"\n", s->name);
                break;
            case TYPE_RUNLEVEL:
                fprintf(fd, "\n#  runlevel \"%s\"\n", s->name);
                break;
            default:
                fprintf(fd, "\n#  unknown  \"%s\"\n", s->name);
                break;
        }

    fprintf(fd, "    ------------------------ \n");

    tmp = s->data;
    while (tmp)
    {
        if (tmp->type)
        {
            if (tmp->type->opt_type == STRING ||
                tmp->type->opt_type == STRINGS
                || tmp->type->opt_type == SCRIPT)
            {

                if (!tmp->t.s)
                {
                F_("empty value!\n")}
                else
                {
                    fprintf(fd, " @@ %s:\t\"%s\"\n", tmp->type->opt_name,
                            tmp->t.s);
                }
            }
            else if (tmp->type->opt_type == INT)
            {
                fprintf(fd, " @@ %s:\t\"%i\"\n", tmp->type->opt_name,
                        tmp->t.i);
            }
        }
        tmp = tmp->next;
    }

    /* if father with data exits, print it */
    if (s->father)
    {
        D_("FATHER FOUND %s\n", s->name);
        service_db_print_u(s->father, fd, FALSE);
    }
    else
        D_("FATHER NOT FOUND %s\n", s->name);

    return;
}

/* Walk through every service and print it all */
void service_db_print_all(FILE * fd)
{

    service_h *current = g.service_cache;

    fprintf(fd, "Full service print:\n");

    D_("print_all():\n");
    while (current)
    {
        service_db_print(current, fd);
        fprintf(fd, "\n\n");

        current = current->next;
    }

    return;
}

static void active_db_print_u(active_h * s, FILE * fd, int intro)
{
    /*data path */
    s_data *tmp = s->data;

    assert(s);
    assert(s->name);

    /* try calculate out how mutch memory we wanna allocate
       that we will send */

    if (intro)
    {
        if (s->from_service)
        {
            switch (s->from_service->type)
            {
                case TYPE_SERVICE:
                    fprintf(fd, "\n#  service  \"%s", s->name);
                    if (s->from_service->father_name)
                        fprintf(fd, " : %s", s->from_service->father_name);
                    fprintf(fd, "\"\n");
                    break;
                case TYPE_CLASS:
                    fprintf(fd, "\n#  class    \"%s\"\n", s->name);
                    break;
                case TYPE_RUNLEVEL:
                    fprintf(fd, "\n#  runlevel \"%s\"\n", s->name);
                    break;
                default:
                    fprintf(fd, "\n#  unknown  \"%s\"\n", s->name);
                    break;
            }
        }
        else
            fprintf(fd, "\n#  virtual  \"%s\"\n", s->name);
    }

    fprintf(fd, "    ------------------------ \n");

    fprintf(fd, "active_db opts:\n");
    tmp = s->data;
    while (tmp)
    {
        if (tmp->type)
        {
            if (tmp->type->opt_type == STRING ||
                tmp->type->opt_type == STRINGS
                || tmp->type->opt_type == SCRIPT)
            {

                if (!tmp->t.s)
                {
                F_("empty value!\n")}
                else
                {
                    fprintf(fd, " @@ %s:\t\"%s\"\n", tmp->type->opt_name,
                            tmp->t.s);
                }
            }
            else if (tmp->type->opt_type == INT)
            {
                fprintf(fd, " @@ %s:\t\"%i\"\n", tmp->type->opt_name,
                        tmp->t.i);
            }
        }
        tmp = tmp->next;
    }

    if (s->from_service)
    {
        fprintf(fd, "service_db opts:\n");
        tmp = s->from_service->data;
        while (tmp)
        {
            if (tmp->type)
            {
                if (tmp->type->opt_type == STRING ||
                    tmp->type->opt_type == STRINGS ||
                    tmp->type->opt_type == SCRIPT)
                {

                    if (!tmp->t.s)
                    {
                    F_("empty value!\n")}
                    else
                    {
                        fprintf(fd, " @@ %s:\t\"%s\"\n", tmp->type->opt_name,
                                tmp->t.s);
                    }
                }
                else if (tmp->type->opt_type == INT)
                {
                    fprintf(fd, " @@ %s:\t\"%i\"\n", tmp->type->opt_name,
                            tmp->t.i);
                }
            }
            tmp = tmp->next;
        }
    }


    fprintf(fd, "service status: %s\n",
            active_db_get_status_string(s->a_status));
    return;
}

/* Walk through every service and print it all */
void active_db_print_all(FILE * fd)
{

    active_h *apt = NULL;

    fprintf(fd, "Full service print:\n");

    D_("print_all():\n");

    apt = g.active_database;
    while (apt)
    {
        active_db_print(apt, fd);
        fprintf(fd, "\n\n");
        apt = apt->next;
    }

    return;
}
