/* Initng, a next generatind sysvinit replacement.
 * Copyright (C) 2005 neuron <neuron@hollowtube.mine.nu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"


#include <stdio.h>
#include <string.h>                         /* strstr() */
#include <stdlib.h>                         /* free() exit() */
#include "../../src/initng_handler.h"
#include "../../src/initng_global.h"
#include "../../src/initng_plugin_hook.h"
#include "../../src/initng_common.h"
#include "../../src/initng_toolbox.h"

#include "initng_syncron.h"

s_entry SYNCRON = { "syncron", STRINGS, 7 };

/* Make sure if service syncron=module_loading, only one of the services with module_loading runs at once */
int check_syncronicly_service(active_h * service)
{
    active_h *tmp = g.active_database;
    char *service_syncron;
    char *tmp_syncron;

    if (service->a_status != START_DEP_MET)
        return (TRUE);

    if ((service_syncron = active_db_get_string(&SYNCRON, service)) == NULL)
        return (TRUE);

    while (tmp)
    {
        /* dont check ourself */
        if (tmp == service)
        {
            tmp = tmp->next;
            continue;
        }
        /* Check every other service in STARTING, see if it has SYNCRON = same_as_us */
        if (tmp->a_status == STARTING)
        {
            if ((tmp_syncron = active_db_get_string(&SYNCRON, tmp)) != NULL)
            {
                if (strcmp(service_syncron, tmp_syncron) == 0)
                {
                    D_("Service %s has to wait for %s\n", service->name,
                       tmp->name);
                    mark_service(service, WAITING_FOR_START_DEP);
                    return (FALSE);
                }
            }
        }
        tmp = tmp->next;
    }
    return (TRUE);
}

/* Make sure there is only one service starting */
int check_syncronicly(active_h * service)
{
    active_h *tmp = g.active_database;

    if (service->a_status != START_DEP_MET)
        return (TRUE);

    while (tmp)
    {
        /* dont check ourself */
        if (tmp == service)
        {
            tmp = tmp->next;
            continue;
        }
        if (tmp->a_status == START_DEP_MET || tmp->a_status == STARTING)
        {
            mark_service(service, WAITING_FOR_START_DEP);
            return (FALSE);
        }
        tmp = tmp->next;
    }
    return (TRUE);
}


int module_init(const char *version)
{
    int i;

    D_("module_init();\n");
    if (strcmp(version, INITNG_VERSION) != 0)
    {
        F_("This module, is compiled for \"%s\" version, and initng is compiled on \"%s\" version, wont load this module!\n", INITNG_VERSION, version);
        return (FALSE);
    }
    initng_sdt_add(&SYNCRON);
    for (i = 0; g.Argv[i]; i++)
        if (strstr(g.Argv[i], "--syncronicly"))
        {
            initng_add_hook(WATCHERS, 10, &check_syncronicly);
            return (TRUE);
        }
    /* Notice this is only added if we dont have --syncronicly */
    D_("Adding syncron\n");
    initng_add_hook(WATCHERS, 10, &check_syncronicly_service);


    return (TRUE);
}

void module_unload(void)
{
    D_("module_unload();\n");
    initng_del_hook(WATCHERS, &check_syncronicly);
    initng_sdt_del(&SYNCRON);
}
