/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "initng.h"
#define _GNU_SOURCE
#include <assert.h>
#include <stdlib.h>
#include <string.h>

#include "initng_control_command.h"

#include "initng_global.h"
#include "initng_toolbox.h"

/* this adds a command to the global command struct */
int initng_command_add(s_command * cmd)
{
    int i = 0;

    assert(cmd);

    /* allocate the big tale if missing */
    if (!g.command_table)
    {
        g.command_table =
            (s_command **) initng_calloc(MAX_COMMANDS, sizeof(s_command *));
        if (!g.command_table)
        {
            F_("Unable to create command table, this is FATAL!\n");
            return (FALSE);
        }
    }

    /* look for duplets */
    for (i = 0; i < MAX_COMMANDS; i++)
    {
        if (g.command_table[i])
            if (cmd == g.command_table[i] ||
                cmd->command_id == g.command_table[i]->command_id)
            {
                F_("Cant add command: %c, %s, it exists alredy!\n",
                   g.command_table[i]->command_id,
                   g.command_table[i]->description);
                return (FALSE);
            }
    }

    /* add on first free spot */
    for (i = 0; i < MAX_COMMANDS; i++)
    {
        if (!g.command_table[i])
        {
            g.command_table[i] = cmd;
            D_("ADDED: %c, %s to command_table[%i]\n", cmd->command_id,
               cmd->description, i);
            return (TRUE);
        }
    }
    return (FALSE);
}

/* this deletes an command from the global command struct */
int initng_command_del(s_command * cmd)
{
    int i = 0;

    assert(cmd);

    /* check that big db is around */
    if (!g.command_table)
    {
        F_("Cant remove command, if command table dont exist!\n");
        return (FALSE);
    }

    /* look for command to delete */
    for (i = 0; i < MAX_COMMANDS; i++)
    {
        if (g.command_table[i] && cmd == g.command_table[i])
        {
            g.command_table[i] = NULL;
            return (TRUE);
        }
    }
    return (FALSE);
}

int initng_command_free_all(void)
{
    if (g.command_table)
        free(g.command_table);
    return (TRUE);
}

/* look for a command by command_id */
s_command *initng_command_find_by_command_id(char cid)
{
    int i = 0;

    /* check that big db is around */
    if (!g.command_table)
    {
        F_("Cant find command, if command table dont exist!\n");
        return (NULL);
    }

    /* look for duplets */
    for (i = 0; i < MAX_COMMANDS; i++)
    {
        if (g.command_table[i] && g.command_table[i]->command_id == cid)
            return (g.command_table[i]);
    }
    return (NULL);
}

/* look for a command by command_id */
s_command *initng_command_find_by_command_string(char *name)
{
    int i = 0;

    /*printf("searching for %s\n", name); */

    /* check that big db is around */
    if (!g.command_table)
    {
        F_("Cant find command, if command table dont exist!\n");
        return (NULL);
    }

    /* look for duplets */
    for (i = 0; i < MAX_COMMANDS; i++)
    {
        if (g.command_table[i])
        {
            /*printf(" * %s\n", g.command_table[i]->long_id); */
            if (strcmp(g.command_table[i]->long_id, name) == 0)
                return (g.command_table[i]);
        }
    }
    return (NULL);
}
