
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "initng.h"

#include <time.h>                           /* time() */
#include <fcntl.h>                          /* fcntl() */
#include <sys/un.h>                         /* memmove() strcmp() */
#include <sys/wait.h>                       /* waitpid() sa */
#include <linux/kd.h>                       /* KDSIGACCEPT */
#include <sys/ioctl.h>                      /* ioctl() */
#include <stdio.h>                          /* printf() */
#include <stdlib.h>                         /* free() exit() */
#include <sys/reboot.h>                     /* reboot() RB_DISABLE_CAD */
#include <assert.h>
#include <errno.h>

#include "initng_global.h"

#include "initng_active_db.h"
#include "initng_toolbox.h"
#include "initng_main.h"
#include "initng_execute.h"
#include "initng_common.h"
#include "initng_load_module.h"

#include "initng_handler.h"
#include "initng_kill_handler.h"

#include "initng_static_data_id.h"

/* function defines */
static int try_to_start_service(active_h * service_to_start);
static int try_to_stop_service(active_h * service_to_stop);
static void set_up_or_done(active_h * service_to_start);


    /* will try start processes waiting for dependencies to go up */
void try_start_wait_deps(void)
{
    active_h *current = g.active_database;
    active_h *next = NULL;

    D_("try_start_wait_deps(void);\n");

    /* walk through all active services */
    while (current)
    {
        assert(current->name);
        D_("try_start_wait_deps(): re-trying with %s.\n", current->name);

        /* we have to remember this, or it might got freed after this */
        next = current->next;

        /* Look at status */
        switch (current->a_status)
        {
            case WAITING_FOR_STOP_DEP:
                /* if succeeded to stop some, remember this */
                try_to_stop_service(current);
                break;
            case WAITING_FOR_START_DEP:
                /* if succeeded to start some, remember this too */
                /* If start dependencies are suddenly met, but we're stopping the system try to stop */
                if (g.sys_state == STATE_STOPPING)
                {
                    try_to_stop_service(current);
                    break;
                }

                try_to_start_service(current);
                break;
            default:
                break;
        }

        current = next;
    }

}

/* this function will send term signal to a service if it was marked 30 seconds or more */
void term_stopping_services(void)
{
    active_h *to_term = g.active_database;
    active_h *next = NULL;

    while (to_term)
    {
        next = to_term->next;

        if ((to_term->a_status == STOPPING ||
             to_term->a_status == STOP_LAUNCHING))
        {
            D_("Service %s is STOPPING\n", to_term->name);
            if (to_term->start_process && to_term->start_process->pid)
            {
                D_("Service %s has a start_process with a valid pid file\n");
                if ((time(0) - to_term->time_got_status) >=
                    SECONDS_BEFORE_KILL)
                {
                    D_("Service %s has been STOPPING for %i, and its more then %i seconds\n", (time(0) - to_term->time_got_status, SECONDS_BEFORE_KILL));
                    /* todo, if kill(process,0) = dead then handle_killed_by_pid without warning and timeout. */

                    /* check if the process still exits! */
                    if (kill(to_term->start_process->pid, 0) < 0
                        && (errno == ESRCH))
                    {
                        /* This process is already dead */
                        F_("process %s pid %d is already dead, trying to fix.\n", to_term->name, to_term->start_process->pid);
                        handle_killed_by_pid(to_term->start_process->pid, 1);
                    }
                    else
                    {

                        W_("Warning, terminating %s on pid %i!\n",
                           to_term->name, to_term->start_process->pid);
                        /* Uhm, this doesnt work : kill(-to_term->start_process->pid, SIGKILL); */
                        kill(to_term->start_process->pid, SIGKILL);
                        /* sigchld might not trigger here, for example if we're using an invalid pidfile (pid = not ours?) */
                        /*                handle_killed_by_pid(to_term->start_process->pid, 1); */
                    }
                }
                else
                {
                    D_("Time is not out ( only %i seconds ) before start killing, waiting %i more seconds.\n", (time(0) - to_term->time_got_status, SECONDS_BEFORE_KILL));
                    alarm(SECONDS_BEFORE_KILL);
                }
            }
        }
        to_term = next;
    }

}

    /* M A R K    S T U F F */

    /* When this function is called, the service is marked for starting */
int start_service(active_h * service_to_start)
{
    assert(service_to_start);
    assert(service_to_start->name);

    /* check system state, if we can launch. */
    if (g.sys_state != STATE_STARTING && g.sys_state != STATE_UP)
    {
        F_("Cant start service, when system status is: %i !\n", g.sys_state);
        return (FALSE);
    }

    /* check if its alredy starting */
    if (service_to_start->a_status == WAITING_FOR_START_DEP)
    {
        D_("service %s is starting allredy.\n", service_to_start->name);
        return (TRUE);
    }

    /* it might actually already be up */
    if (service_to_start->a_status == DONE ||
        service_to_start->a_status == RUNNING)
    {
        D_("service %s is alredy up!\n", service_to_start->name);
        return (TRUE);
    }

    /* if service has not started stopping yet, it can be set to up directly */
    if (service_to_start->a_status == WAITING_FOR_STOP_DEP ||
        service_to_start->a_status == STOP_DEP_MET)
    {
        set_up_or_done(service_to_start);
        D_("service %s did never have tome to stop!\n",
           service_to_start->name);
        return (TRUE);
    }

    /* STEP 2, service must be in one of these statuses to start */
    if (service_to_start->a_status != LOADING &&
        service_to_start->a_status != STOPPED &&
        service_to_start->a_status != RESPAWNING &&
        service_to_start->a_status != RESTARTING)
    {
        W_("cant set a service with status %i:%s, to start\n",
           service_to_start->a_status,
           active_db_get_status_string(service_to_start->a_status));
        return (FALSE);
    }

    /* make sure try_to_start_wait_deps is run on main loop */
    g.interrupt = TRUE;

    /* mark it WAITING_FOR_START_DEP and wait */
    if (!mark_service(service_to_start, WAITING_FOR_START_DEP))
    {
        W_("mark_service WAITING_FOR_START_DEP failed for service %s\n",
           service_to_start->name);
        return (FALSE);
    }
    return (TRUE);
}

    /* S T O P     S E R V I C E */
int stop_service(active_h * service_to_stop)
{
    active_h *current = NULL;

    assert(service_to_stop);

    D_("stop_service(%s);\n", service_to_stop->name);

    /* if service is alredy stoped, return */
    if (service_to_stop->a_status == STOPPED)
        return (TRUE);

    /* this can be a cirular dep, if then return */
    if (service_to_stop->a_status == STOP_MARKED ||
        service_to_stop->a_status == WAITING_FOR_STOP_DEP)
        return (TRUE);

    /* if service are waiting to start, there is no problem to shut it down directly */
    if (service_to_stop->a_status == WAITING_FOR_START_DEP ||
        service_to_stop->a_status == START_DEP_MET)
    {
        if (!mark_service(service_to_stop, STOP_DEP_MET))
            return (FALSE);
        return (TRUE);
    }

    /* check if service status, is really up */
    if (service_to_stop->a_status != DONE &&
        service_to_stop->a_status != RUNNING)
        W_("Warning, setting service %s, status %s, to STOP_MARKED!\n",
           service_to_stop->name,
           active_db_get_status_string(service_to_stop->a_status));

    /* set stopping */
    if (!mark_service(service_to_stop, STOP_MARKED))
        return (FALSE);

    /* mark all services depending on this service stop */
    while ((current = active_db_next_dep_on_me(service_to_stop, current)))
    {
        D_("%s have to stop %s.\n", service_to_stop->name, current->name);
        stop_service(current);
    }

    /* make sure, main loop tries to start WAITING_FOR_STARTDEP */
    g.interrupt = TRUE;

    /* Mark this service, that is is waiting for its stop deps to stop */
    if (!mark_service(service_to_stop, WAITING_FOR_STOP_DEP))
        return (FALSE);

    return (TRUE);
}

int restart_service(active_h * service_to_restart)
{
    active_db_mark(service_to_restart, MARK_RESTARTING);
    return (stop_service(service_to_restart));
}



/* Load new process named */
active_h *load_new_service_named(const char *service)
{
    active_h *to_load = NULL;

    D_("%s\n", service);

    assert(service);

    if (!(to_load = load_to_active(service)))
    {
        /* the function calling this function will print out an error */
        /*F_("Unable to load active for service %s\n", service) */
        return (NULL);
    }

    /* STEP 3, mark service loading, if new */
    if ((to_load->a_status == NONE) && !mark_service(to_load, LOADING))
    {
        D_("Mark service failed!\n");
        return (NULL);
    }

    return (to_load);
}

    /* start new process named */
int start_new_service_named(const char *service)
{
    active_h *to_start = NULL;

    assert(service);
    D_("%s\n", service);

    if (!(to_start = load_new_service_named(service)))
    {
        F_("Unable to load servie %s!\n", service);
        return (FALSE);
    }

    return (start_service(to_start));
}

    /* enter a new runlevel */
int stop_all(void)
{
    active_h *ser = g.active_database;

    set_sys_state(STATE_STOPPING);
    D_("stop_all();\n");
    while (ser)
    {
        stop_service(ser);
        ser = ser->next;
    }
    return (TRUE);
}





/* These two functions will stop services that are not dependencies 
   of the current runlevel */
static void mark_deps(active_h * h)
{
    W_("mark_deps(%s) Not implented!\n", h->name);
}

void stop_unneeded_services(void)
{
    active_h *s = NULL;

    /* Remove mark from all active services */
    active_db_unmark_all(MARK_NEEDED);

    /* Recursively mark all dependencies of the current runlevel */
    s = active_db_find_by_name(g.runlevel);
    if (!s)
        return;
    mark_deps(s);

    /* Now rewalk the tree, stopping anything not marked as needed */
    s = g.active_database;
    while (s)
    {
        if (!active_db_is_m(s, MARK_NEEDED))
        {
            stop_service(s);
        }
        s = s->next;
    }
}



    /* D O    S T U F F */


    /* This is the function that really launches the start-marked services if all deps are up */
static int try_to_start_service(active_h * service_to_start)
{
    assert(service_to_start);

    D_("try_to_start_service(%s): status: \"%s\"\n",
       service_to_start->name,
       active_db_get_status_string(service_to_start->a_status));

    if (g.sys_state != STATE_STARTING && g.sys_state != STATE_UP)
    {
        F_("Cant start service, when system status is: %i !\n", g.sys_state);
        return (FALSE);
    }

    /* set status to START_DEP_MET */
    if (!mark_service(service_to_start, START_DEP_MET))
        return (FALSE);

    /* check if it has a start process */
    if (service_to_start->start_process)
    {
        F_("try_to_start_service(%s): Function already has a start process! Aborting\n", service_to_start->name);
        return (FALSE);
    }

    /* F I N A L L Y   S T A R T */

    switch (start_launch(service_to_start))
    {
        case FALSE:
            D_("Done direct!\n");
            /* D O N E - D I R E C T */
            if (!mark_service(service_to_start, DONE))
                return (FALSE);
            return (TRUE);                  /* return with status */
        case FAIL:
            F_("Did not find START file!\n");
            mark_service(service_to_start, FAIL_STARTING);
            return (FALSE);                 /* return with status */

    }

    /* make sure there are a start process */
    if (!service_to_start->start_process)
    {
        mark_service(service_to_start, FAIL_STARTING);
        return (FALSE);                     /* return with status */
    }

    D_("Launch Started.\n");
    set_up_or_done(service_to_start);
    if (service_to_start->a_status == FAIL_STARTING)
        return (FALSE);
    return (TRUE);
}

    /* try to find out if service is up or done */
static void set_up_or_done(active_h * service_to_start)
{
    /* S E T   A   S T A T U S */

    /* if marked that service up when done or pid set, mark this
       service as starting */
    if (active_db_is(&UP_WHEN_DONE, service_to_start) ||
        active_db_is(&UP_WHEN_PID_SET, service_to_start))
    {
        D_("Found UP_WHEN_DONE or UP_WHEN_PID_SET, seting status to starting.\n");
        if (!mark_service(service_to_start, STARTING))
            return;
        return;
    }


    /* if UP_WHEN_STARTED set, set status and return */
    if (active_db_is(&UP_WHEN_STARTED, service_to_start) ||
        active_db_is(initng_sdt_find("daemon"), service_to_start) ||
        active_db_is(initng_sdt_find("daemon {"), service_to_start))
    {
        D_("Found UP_WHEN_STARTED or DAEMON, seting status to running.\n");
        if (!mark_service(service_to_start, RUNNING))
            return;
        return;
    }


    /* by default, return service starting */
    D_("defaulting to STARTING\n");
    if (!mark_service(service_to_start, STARTING))
        return;
    return;                                 /* return with status */
}


    /* do stop service */
static int try_to_stop_service(active_h * service_to_stop)
{
    active_h *current = NULL;

    assert(service_to_stop);

    D_("try_to_stop_service(%s): status :\"%s\"\n",
       service_to_stop->name,
       active_db_get_status_string(service_to_stop->a_status));

    /* if there are services depending on this one still running, return false and still try */
    while ((current = active_db_next_dep_on_me(service_to_stop, current)))
    {
        if (current->a_status == DONE ||
            current->a_status == RUNNING ||
            current->a_status == STOP_MARKED ||
            current->a_status == STOPPING ||
            current->a_status == WAITING_FOR_STOP_DEP ||
            current->a_status == STOP_DEP_MET)
        {
            D_("There is deps, depending on %s still, waiting a bit more\n",
               service_to_stop->name);
            return (FALSE);                 /* wait */
        }
    }

    /* ok, stopping deps are met */
    if (!mark_service(service_to_stop, STOP_DEP_MET))
    {
        D_("Cound not set status of %s to STOP_DEP_MET.\n",
           service_to_stop->name);
        return (FALSE);
    }

    /* check if it has a stop process */
    if (service_to_stop->stop_process)
    {
        F_("try_to_stop_service(%s): Function already have a stop process! Aborting!\n", service_to_stop->name);
        mark_service(service_to_stop, FAIL_STOPPING);
        return (FALSE);
    }

    /* F I N A L L Y   S T O P */
    if (!mark_service(service_to_stop, STOPPING))
    {
        D_("Cound not set status of %s to STOPPING.\n",
           service_to_stop->name);
        return (FALSE);
    }

    /* launch stop service */
    if (stop_launch(service_to_stop) == FAIL)
    {
        F_("try_to_stop_service(%s): fail launch stop!\n",
           service_to_stop->name);
        mark_service(service_to_stop, FAIL_STOPPING);
        return (FALSE);
    }


    /* if we got a stop process with a pid */
    if (service_to_stop->stop_process && service_to_stop->stop_process->pid)
    {
        alarm(SECONDS_BEFORE_KILL + 1);     /* the alarm will try to kill stopping services */
        D_("Clock is set, %s stop script, have %i seconds to kill the service.\n", service_to_stop->name, SECONDS_BEFORE_KILL);
        return (TRUE);                      /* return from here */
    }

    /* T R Y 2 , Kill start_process directly */
    if (service_to_stop->start_process
        && service_to_stop->start_process->pid > 0)
    {
        D_("try_to_stop(%s): REAL KILLING PID %i!\n",
           service_to_stop->name, service_to_stop->start_process->pid);

        /* Uhm, this doesnt work : kill(-service_to_stop->start_process->pid, SIGTERM); */
        kill(service_to_stop->start_process->pid, SIGTERM);

        alarm(SECONDS_BEFORE_KILL + 1);     /* the alarm will try to kill stopping services */
        D_("Clock is set, killed process %i (%s), have %i seconds to quit before SIGKILL\n", service_to_stop->start_process->pid, service_to_stop->name, SECONDS_BEFORE_KILL);
        /*kill(service_to_stop->start_process->pid,SIGKILL); */
        return (TRUE);
    }
    /* if this is a empty dep service we show up here */

    /* remember this in history */
    if (!mark_service(service_to_stop, STOPPED))
    {
        D_("Coud not set status STOPPED!\n");
        return (FALSE);
    }

    /* check if this service is restarting */
    if (active_db_is_m(service_to_stop, MARK_RESTARTING))
    {
        if (!mark_service(service_to_stop, RESTARTING))
            return (FALSE);
        active_db_unmark(service_to_stop, MARK_RESTARTING);
        start_service(service_to_stop);
        D_("Service is restarting now!\n");
        return (TRUE);
    }


    /* free service, and forget */
    active_db_del(service_to_stop);
    D_("Service removed.\n");
    return (TRUE);
}
