
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "initng.h"

#include <time.h>                           /* time() */
#include <fcntl.h>                          /* fcntl() */
#include <sys/un.h>                         /* memmove() strcmp() */
#include <sys/wait.h>                       /* waitpid() sa */
#include <linux/kd.h>                       /* KDSIGACCEPT */
#include <sys/ioctl.h>                      /* ioctl() */
#include <stdlib.h>                         /* free() exit() */
#include <sys/reboot.h>                     /* reboot() RB_DISABLE_CAD */
#include <termios.h>
#include <stdio.h>
#include <sys/klog.h>

#include "initng_global.h"
#include "initng_signal.h"
#include "initng_history.h"
#include "initng_handler.h"
#include "initng_execute.h"                 /* new_environ() */
#include "initng_active_db.h"
#include "initng_load_module.h"
#include "initng_plugin_callers.h"
#include "initng_toolbox.h"


#include "initng_main.h"
/*extern char **environ; */

/*
 * Poll if there are any processes left, and if not quit initng
 */
void ready_to_quit(void)
{
    D_("ready_to_quit(): %i process(es) remaining.\n", active_db_count(0));
    /* Track the count of remaining processes */
#ifndef DEBUG
    if (g.sys_state == STATE_STOPPING)
    {
        active_h *tmp = g.active_database;

        while (tmp)
        {
            D_("ready_to_quit(): ********  %s still active (status: %s)  *******\n", tmp->name, active_db_get_status_string(tmp->a_status));
            tmp = tmp->next;
        }
    }
#endif


    /* If the last process has died, quit initng */
    if ((active_db_count(NONE)) < 1)
    {
        set_sys_state(STATE_ASE);
        printf
            ("\n\n\n    **********        No more services!        ************\n\n");
        /* do this in shutdown_script instead 
           if (g.i_am_init)
           {
           W_("Sending all processes the TERM signal, and wait 2 sec...\n");
           kill(-1, SIGTERM);
           usleep(2000000);
           W_("Sending all processes the KILL signal...\n");
           kill(-1, SIGKILL);
           }
         */

#ifdef DEBUG
        if (g.runlevel)
            D_("ready_to_quit(): Restarting from service/runlevel: %s\n\n",
               g.runlevel);
#endif
        switch (g.when_out)
        {
            case THEN_QUIT:
                exit_initng(0);
                break;
            case THEN_RESTART:
                restart_initng();
                break;
            case THEN_SULOGIN:
                su_login();
                break;
            case THEN_START_DEFAULT:
            case THEN_NEW_RUNLEVEL:
                set_sys_state(STATE_STARTING);
                if (g.runlevel)
                {
                    /* if runlevel is set to sulogin, do that */
                    if ((strcasecmp(g.runlevel, "sulogin") == 0))
                        su_login();
                    else
                        start_new_service_named(g.runlevel);
                }
                else
                    restart_initng();
                break;
            case THEN_NONE:
            default:
                F_("Error: invalid command (code %d)\n", g.when_out);
                /* FALLTHROUGH */
        }
    }
}

/*
 * Exiting on an initialization failure is not a good idea for init
 */
void su_login(void)
{
    set_sys_state(STATE_SULOGIN);
    char *argv[] = { i_strdup("sulogin"), NULL };
    execve("/sbin/sulogin", argv, new_environ(NULL));   /* launch sulogin */
    _exit(1);
}

/*
 * Start initial services
 */
void start_extra_services(void)
{
    int i;
    int a_count = 0;            /* counts orders from argv to start */

    set_sys_state(STATE_STARTING);
    /* check with argv which service to start initiating */
    for (i = 1; i < g.Argc; i++)
    {
        /* start all services that is +service */
        if (g.Argv[i][0] != '+')
            continue;

        /* if succeed to load this service */
        if (start_new_service_named(g.Argv[i] + 1))
            a_count++;
    }
}


void restart_initng(void)
{
    set_sys_state(STATE_RESTART);
    fprintf(stderr,
            "\n\n\n          R E S T A R T I N G,  (Really hot reboot)\n\n");
    execve("/sbin/initng", g.Argv, environ);
}

void sys_state_services_loaded(void)
{
    h_sys_state oldstate = g.sys_state;

    set_sys_state(STATE_SERVICES_LOADED);
    g.sys_state = oldstate;
    return;
}


/* this function sets g.sys_state, and call plugins that listen on its change */
void set_sys_state(h_sys_state state)
{
    /* don't set a state that is */
    if (state == g.sys_state)
        return;

    D_("set_sys_state(): %% Setting state to: %i %% \n", state);
    g.sys_state = state;

    /* execute all functions in modules that want to
       be executed when system state change occurs */

    initng_load_module_system_changed(state);
    return;
}


void exit_initng(int i)
{
    D_("exit_initng();\n");

    set_sys_state(STATE_EXIT);

    initng_unload_all_modules();
    initng_free();
    exit(i);
}

/*
   int blacklist_add(const char *sname) {
   int i=0;
   if(!g.blacklist)
   initng_calloc(MAX_BLACKLIST, sizeof(char *));

   for(i=0;(i<MAX_BLACKLIST && g.blacklist[i]) ;i++) i++;

   if(i>=MAX_BLACKLIST) return(FALSE);

   g.blacklist[i]=i_strdup(sname);
   return(TRUE);
   } */
