
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <assert.h>
#include <time.h>

#include "initng.h"

#include "initng_global.h"

#include "initng_toolbox.h"
#include "initng_plugin_hook.h"
#include "initng_plugin.h"

/* #define initng_del_hook(t,h) if (g.call_db); g.call_db = initng_del_hook_real(g.call_db,t,h); */

/* deletes hook by hook type, and pointer */
s_call *initng_del_hook_real(s_call * db, e_ptype t, void *hook)
{
    s_call *next = NULL;

    if (db->ptype == t && db->c.hook == hook)
    {
        next = db->next;
        free(db);
        return next;
    }
    if (db->next)
        db->next = initng_del_hook_real(db->next, t, hook);
    return db;
}

static s_call *initng_add_hook_real(s_call * db, e_ptype t, int order,
                                    void *hook, int fd, int seconds)
{
    s_call *next = NULL;

    /* We haven't found a place to inject ourself, so add to end of database */
    if (!db)
    {
        D_("Allocing new plugin caller\n");
        db = initng_calloc(1, sizeof(s_call));
        goto add_and_return;
    }

    /* Inject ourself and stop scanning */
    if (db->order > order)
    {
        D_("Injecting plugin caller\n");
        next = db;
        db = initng_calloc(1, sizeof(s_call));
        db->next = next;
        goto add_and_return;
    }

    /* Nothing found, just continue the scan */
    db->next = initng_add_hook_real(db->next, t, order, hook, fd, seconds);
    return db;

  add_and_return:
    switch (t)
    {
        case FDWATCHERS:
            db->c.fdh.call_module = hook;
            db->c.fdh.fds = fd;
            D_("ADDED FD %i\n", fd);
            break;
        case TIMERS:
            db->c.tdh.call_module = hook;
            db->c.tdh.seconds = seconds;
            db->c.tdh.next_hit = time(NULL) + seconds;
            alarm(seconds);
            D_("ADDED TIMER %i (%i)\n", seconds, db->c.tdh.next_hit);
            break;
        default:
            db->c.hook = hook;
            break;
    }
    db->ptype = t;
    db->order = order;
    D_("hook added.\n");
    return db;
}

int initng_add_hook(e_ptype t, int order, void *hook, ...)
{
    va_list list;
    int fd = 0;
    long seconds = 0;

    assert(hook);
    D_("Adding hook type %i\n", t);

    /* Check parameters */
    if (t == FDWATCHERS || t == TIMERS)
    {
        va_start(list, hook);

        if (t == FDWATCHERS)
        {
            fd = va_arg(list, int);

            if (fd < 3)
            {
                F_("Filedescriptor must be >= 3.\n");
                return FALSE;
            }
        }
        if (t == TIMERS)
            seconds = va_arg(list, long);

        va_end(list);
    }

    g.call_db = initng_add_hook_real(g.call_db, t, order, hook, fd, seconds);
    return (TRUE);
}
