
/* Initng, a next generatind sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "initng.h"

#include <string.h>
#include <stdlib.h>
#include <assert.h>

#include "initng_service_data_types.h"
#include "initng_global.h"
#include "initng_toolbox.h"
#include "initng_static_data_id.h"

s_entry *initng_sdt_add(s_entry * ent)
{
    int i = 0;

    D_("attempting to add \"%s\" type %i to option_table\n", ent->opt_name,
       ent->opt_type);

    /* Check that option table exists */
    if (!g.option_table)
    {
        D_("Allocating option table!\n");
        g.option_table =
            (s_entry **) initng_calloc(MAX_DATA_TYPES, sizeof(s_entry *));
        if (!g.option_table)
        {
            F_("UNABLE TO CREATE OPTION_TABLE!, THIS IS FATAL!\n");
            return (NULL);
        }
    }
    /* it might already have been added */
    {
        s_entry *tmp = NULL;

        if ((tmp = initng_sdt_find(ent->opt_name)))
        {
            W_("Won't add %s again!\n", ent->opt_name);
            return (tmp);
        }
    }

    /* fast forward to a free spot */
    while (i < MAX_DATA_TYPES && g.option_table[i])
    {
        i++;
    }

    if (i == MAX_DATA_TYPES)
    {
        F_("Reached MAX_DATA_TYPES!\n");
        return (NULL);
    }

    D_("free spot at %i to add service on\n", i);

    g.option_table[i] = ent;

    D_(" \"%s\" added to option_table!\n", ent->opt_name);
    return (g.option_table[i]);
}

void initng_sdt_del(s_entry * ent)
{
    int i;
    active_h *currentA = g.active_database;
    service_h *currentS = g.service_cache;

    D_("Freeing everything associated with entry\n");

    /* clear the active db for this data */
    while (currentA)
    {
        d_remove(ent, currentA->data);
        currentA = currentA->next;
    }

    /* clear the file cache from this data */
    while (currentS)
    {
        d_remove(ent, currentS->data);
        currentS = currentS->next;
    }

    D_("Clearing entry\n");
    for (i = 0; i < MAX_DATA_TYPES; i++)
    {
        if (g.option_table[i] == ent)
        {
            g.option_table[i] = NULL;
        }
    }
}

void initng_sdt_free_all(void)
{
    int i;

    /* just reset all entries */
    for (i = 0; i < MAX_DATA_TYPES; i++)
    {
        g.option_table[i] = NULL;
    }
    free(g.option_table);
    g.option_table = NULL;
}

s_entry *initng_sdt_find(const char *string)
{
    int i;

    assert(string);
    D_("looking for %s.\n", string);
    for (i = 0; i < MAX_DATA_TYPES; i++)
        if (g.option_table[i] && g.option_table[i]->opt_name
            && strcasecmp(g.option_table[i]->opt_name, string) == 0)
            return (g.option_table[i]);

    return (NULL);
}
