#include <CUnit/CUnit.h>
#include <CUnit/Basic.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include "test_initng.h"

#include "../src/initng_process_db.h"

/* 
 * Create a new process_db entry and verify that it is initialised 
 * properly. 
 */

static void test_process_db_new(void)
{
    process_h *p;
    char *w_buf = "garbage in, garbage out\n";
    char r_buf[128];

    process_db_new(&p, T_START);

    /* basic checks */
    CU_ASSERT(p->pt == T_START);
    CU_ASSERT(p->pid == 0);
    CU_ASSERT(p->r_code == -10);            /* XXX: magic number */
    CU_ASSERT(p->buffer != NULL);
    CU_ASSERT(p->buffer_pos == 0);

    /* check that our pipe works */
    CU_ASSERT(write(p->stdout[1], w_buf, strlen(w_buf) + 1) ==
              strlen(w_buf) + 1);
    CU_ASSERT(read(p->stdout[0], r_buf, sizeof(r_buf)) == strlen(w_buf) + 1);
    CU_ASSERT(strcmp(w_buf, r_buf) == 0);

    process_db_free(&p);
}

/* 
 * verify process_db_free() works
 */

/* see if the given integer is a file descriptor */
static int is_fd(int fd)
{
    struct stat buf;

    if (fstat(fd, &buf) == -1)
    {
        return 0;
    }
    else
    {
        return 1;
    }
}

static void test_process_db_free(void)
{
    process_h *p;
    int fds[2];

    /* hope we don't core dump */
    p = NULL;
    process_db_free(&p);

    /* standard free */
    process_db_new(&p, T_START);
    fds[0] = p->stdout[0];
    fds[1] = p->stdout[1];
    process_db_free(&p);

    /* check our result */
    CU_ASSERT(p == NULL);

    /* verify our pipes have closed */
    CU_ASSERT(!is_fd(fds[0]));
    CU_ASSERT(!is_fd(fds[1]));

    /* try a double-free */
    process_db_free(&p);
}

void test_process_db_setup(void)
{
    CU_pSuite suite;

    suite = CU_add_suite("initng_processdb", empty_init, empty_cleanup);
    CU_add_test(suite, "process_db_new", test_process_db_new);
    CU_add_test(suite, "process_db_free", test_process_db_free);
}
